{******************************************************************************}
{                        --Excel Class-- (Excel95͎gps)                   }
{******************************************************************************}
unit Excel;

interface

uses
  Windows, Messages, SysUtils, Graphics, ComObj, Variants, Grids, ComCtrls;


type
  TAlign   = 2..4;
  TPattern = -4166..18;

const
  {Align}
   xlAlignLeft   = TAlign(2);
   xlAlignCenter = TAlign(3);
   xlAlignRight  = TAlign(4);

  {LineWidth}
   xlLW_Hairline = 1;     //א
   xlLW_Thin     = 2;     //W
   xlLW_Medium   = -4138; //
   xlLW_Thick    = 4;     //ɑ
  {LineStyle}
   xlLS_Continuous    = 1;
   xlLS_Dash          = -4115;
   xlLS_DashDot       = 4;
   xlLS_DashDotDot    = 5;
   xlLS_Dot           = -4118;
   xlLS_Double        = -4119;
   xlLS_LineStyleNone = -4142;
   xlLS_SlantDashDot  = 13;
  {Borders}
   xlDiagonalDown     = 5;
   xlDiagonalUp       = 6;
   xlEdgeLeft         = 7;
   xlEdgeTop          = 8;
   xlEdgeBottom       = 9;
   xlEdgeRight        = 10;
   xlinsideVartical   = 11;
   xlinsideHorizontal = 12;
  {PrintPreview}
   xlPreviewNormal =1;
   xlPreviewWide   =2;
  {FileFormatType}
   xlCSV                 = 6;
   xlUnicodeText         = 42;
   xlExcel9795           = 43;
   xlCurrentPlatformText = -4158;
  {Pattern}
   xlPatternAutomatic       = TPattern(-4105);
   xlPatternChecker         = TPattern(9);
   xlPatternCrissCross      = TPattern(16);
   xlPatternDown            = TPattern(-4121);
   xlPatternGray8           = TPattern(18);
   xlPatternGray16          = TPattern(17);
   xlPatternGray25          = TPattern(-4124);
   xlPatternGray50          = TPattern(-4125);
   xlPatternGray75          = TPattern(-4126);
   xlPatternGrid            = TPattern(15);
   xlPatternHorizontal      = TPattern(-4128);
   xlPatternLightDown       = TPattern(13);
   xlPatternLightHorizontal = TPattern(11);
   xlPatternLightUp         = TPattern(14);
   xlPatternLightVertical   = TPattern(12);
   xlPatternNone            = TPattern(-4142);
   xlPatternSemiGray75      = TPattern(10);
   xlPatternSolid           = TPattern(1);
   xlPatternUp              = TPattern(-4162);
   xlPatternVertical        = TPattern(-4166);
  {GraphKind}//Add Ver1.0.0.4
   xlBar         = 2;
   xlColumn      = 3;
   xlLine        = 4;
   xlPie         = 5;
   xlRadar       = -4151;
   xlXYScatter   = -4169;
   xlCombination = -4111;
   xl3DArea      = -4098;
   xl3Dbar       = -4099;
   xl3DColumn    = -4100;
   xl3DLine      = -4101;
   xl3DPie       = -4102;
   xl3DSurface   = -4103;
   xlDoughnut    = -4120;


type
  EWorkSheetError = class(Exception);
  TWorkSheet = class
  private
     E_WorkSheet: Variant;
     E_Charts   : Variant;
     FFormat  : string;
     FColor   : TColor;
     FPattern : TPattern;
     FFont    : TFont;
     FAlign   : TAlign;
     FCol,FRow: integer;
     function GetCells( ACol, ARow: Integer ): Variant;
     function GetName:string;
     procedure SetCells( ACol, ARow: Integer; const Value: Variant );
     procedure SetName(NewName:string);
     function Alpha(I: integer):string;
     function ExcelRange(Rect:TRect):string;
  public
     constructor Create( WorkSheet:Variant );
     destructor Destroy;override;
     procedure SetStyleCell( Col,Row:integer );
     procedure SetStyleRect( Rect:TRect );
     procedure AutoFit( ColCount:integer );
     procedure Activate;
     procedure PrintPreview;
     procedure SetHeader( strL,strC,strR:string );
     procedure SetFooter( strL,strC,strR:string );
     procedure GridToExcel( SG:TStringGrid );
     procedure ListToExcel( LV:TListView );
     procedure MakeTableTemplate( ColumnRect,ItemRect:TRect; LineWidth:integer );
     procedure MakeGraph( Rect:TRect ;GraphKind:integer ;HasLegend:Boolean ;Title:string );
     property Cells[ACol, ARow: Integer]: Variant read GetCells write SetCells;
     property Name    :string   read GetName  write SetName;
     property Format  :string                 write FFormat;
     property Color   :TColor   read FColor   write FColor;
     property Pattern :TPattern read FPattern write FPattern;
     property Font    :TFont    read FFont    write FFont;
     property Align   :TAlign   read FAlign   write FAlign;
     property Sheet   :Variant  read E_WorkSheet;
  end;


type
  EExcelError = class(Exception);
  TExcel = class
  private
     E_Excel       : Variant;
     E_Application : Variant;
     E_WorkBook    : Variant;
     function GetSheetsCount:integer;
     function GetPrinterName:string;
     function GetHeight:integer;
     function GetWidth:integer;
     function GetTop:integer;
     function GetLeft:integer;
     procedure SetHeight(Height:integer);
     procedure SetWidth(Width:integer);
     procedure SetTop(Top:integer);
     procedure SetLeft(Left:integer);
  public
     WorkSheets  : array of TWorkSheet;     //ׂĂSheetzێ
     constructor Create(FileName:string='');
     destructor Destroy;override;
     procedure  ExcelClose;
     procedure  SaveAs(FileName:string;FileFormatType:integer);
     property SheetsCount : integer read GetSheetsCount;
     property PrinterName : string  read GetPrinterName;
     property Top         : integer read GetTop            write SetTop;
     property Left        : integer read GetLeft           write SetLeft;
     property Height      : integer read GetHeight         write SetHeight;
     property Width       : integer read GetWidth          write SetWidth;
  end;


implementation


{ TWorkSheet }

function TWorkSheet.GetCells(ACol, ARow: Integer): Variant;
begin
   try
      Result := E_WorkSheet.Cells[ARow,ACol].Value;
   except
      raise EWorkSheetError.Create('WorkSheet Read Error. msg=Cell['+IntToStr(ACol)+','+IntToStr(ARow)+']͔͈͊Oł');
   end;
end;

procedure TWorkSheet.SetCells(ACol, ARow: Integer; const Value: Variant);
begin
   try
      E_WorkSheet.Cells[ARow,ACol].Value := Value;
   except
      raise EWorkSheetError.Create('WorkSheet Write Error. msg=Cell['+IntToStr(ACol)+','+IntToStr(ARow)+']͔͈͊Oł');
   end;
end;

function TWorkSheet.GetName: string;
begin
   Result := E_WorkSheet.Name;
end;

procedure TWorkSheet.SetName(NewName: string);
begin
   E_WorkSheet.Name := NewName;
end;

{*******************************************************************************
mn
	ɑΉExcelp̃At@xbgԂ
mn
	I :integer
m߂ln
        string
*******************************************************************************}
function TWorkSheet.Alpha(I: integer): string;
begin
   case I of
       1 : Result :='A';   2 : Result :='B';    3 : Result :='C';
       4 : Result :='D';   5 : Result :='E';    6 : Result :='F';
       7 : Result :='G';   8 : Result :='H';    9 : Result :='I';
       10: Result :='J';   11: Result :='K';    12: Result :='L';
       13: Result :='M';   14: Result :='N';    15: Result :='O';
       16: Result :='P';   17: Result :='Q';    18: Result :='R';
       19: Result :='S';   20: Result :='T';    21: Result :='U';
       22: Result :='V';   23: Result :='W';    24: Result :='X';
       25: Result :='Y';   26: Result :='Z';    27: Result :='AA';
       28: Result :='AB';  29: Result :='AC';   30: Result :='AD';
       31: Result :='AE';  32: Result :='AF';   33: Result :='AG';
       34: Result :='AH';  35: Result :='AI';   36: Result :='AJ';
       37: Result :='AK';  38: Result :='AL';   39: Result :='AM';
       40: Result :='AN';  41: Result :='AO';   42: Result :='AP';
       43: Result :='AQ';  44: Result :='AR';   45: Result :='AS';
       46: Result :='AT';  47: Result :='AU';   48: Result :='AV';
       49: Result :='AW';  50: Result :='AX';   51: Result :='AY';
       52: Result :='AZ';  53: Result :='BA';   54: Result :='BB';
       55: Result :='BC';  56: Result :='BD';   57: Result :='BE';
       58: Result :='BF';  59: Result :='BG';   60: Result :='BH';
       else
         Result := '';
   end;
end;
{*******************************************************************************
mn
	ExcelFormatRangeԂ
mn
	Rect :TRect [FRect(1,1,9,5)]
m߂ln
        string      [F"A1:I5"]
*******************************************************************************}
function TWorkSheet.ExcelRange(Rect: TRect): string;
begin
   Result := Alpha(Rect.Left)+IntToStr(Rect.Top)+':'+Alpha(Rect.Right)+IntToStr(Rect.Bottom);
end;


constructor TWorkSheet.Create(WorkSheet: Variant);
begin
   inherited Create;
   FFont := TFont.Create;
   FFont.Name := 'MS SVbN';
   FFont.Size := 9;
   FFont.Style := [];
   E_WorkSheet  := WorkSheet;
   FColor   := clWhite;
   FPattern := xlPatternSolid;
   FAlign   := xlAlignRight;
   FCol := 1;
   FRow := 1;
end;

destructor TWorkSheet.Destroy;
begin
   E_WorkSheet := Unassigned;
   E_Charts  := Unassigned;
   FFont.Free;
   inherited Destroy;
end;

procedure TWorkSheet.SetStyleCell(Col, Row: integer);
begin
   try
       E_WorkSheet.Cells[Row,Col].Font.Size  := FFont.Size;
       E_WorkSheet.Cells[Row,Col].Font.Name  := FFont.Name;
       E_WorkSheet.Cells[Row,Col].Font.Color := FFont.Color;
       if (fsBold      in FFont.Style) then E_WorkSheet.Cells[Row,Col].Font.Bold := True;
       if (fsItalic    in FFont.Style) then E_WorkSheet.Cells[Row,Col].Font.Italic := True;
       if (fsUnderline in FFont.Style) then E_WorkSheet.Cells[Row,Col].Font.UnderLine := True;
       if (fsStrikeout in FFont.Style) then E_WorkSheet.Cells[Row,Col].Font.Strikethrough := True;
       E_WorkSheet.Cells[Row,Col].HorizontalAlignment := FAlign;
       E_WorkSheet.Cells[Row,Col].Interior.Color      := FColor;
       E_WorkSheet.Cells[Row,Col].Interior.Pattern    := FPattern;
       E_WorkSheet.Cells[Row,Col].NumberFormat        := FFormat;
   except
      on E:Exception do begin
         raise EWorkSheetError.Create('WorkSheet SetStyleCell Error. msg='+E.Message);
      end;
   end;
end;

procedure TWorkSheet.SetStyleRect(Rect: TRect);
begin
   try
       E_WorkSheet.Range[ExcelRange(Rect)].Font.Size  := FFont.Size;
       E_WorkSheet.Range[ExcelRange(Rect)].Font.Name  := FFont.Name;
       E_WorkSheet.Range[ExcelRange(Rect)].Font.Color := FFont.Color;
       if (fsBold      in FFont.Style) then E_WorkSheet.Range[ExcelRange(Rect)].Font.Bold := True;
       if (fsItalic    in FFont.Style) then E_WorkSheet.Range[ExcelRange(Rect)].Font.Italic := True;
       if (fsUnderline in FFont.Style) then E_WorkSheet.Range[ExcelRange(Rect)].Font.UnderLine := True;
       if (fsStrikeout in FFont.Style) then E_WorkSheet.Range[ExcelRange(Rect)].Font.Strikethrough := True;
       E_WorkSheet.Range[ExcelRange(Rect)].HorizontalAlignment := FAlign;
       E_WorkSheet.Range[ExcelRange(Rect)].Interior.Color      := FColor;
       E_WorkSheet.Range[ExcelRange(Rect)].Interior.Pattern    := FPattern;
       E_WorkSheet.Range[ExcelRange(Rect)].NumberFormat        := FFormat;
   except
      on E:Exception do begin
         raise EWorkSheetError.Create('WorkSheet SetStyleCell Error. msg='+E.Message);
      end;
   end;
end;

procedure TWorkSheet.AutoFit(ColCount: integer);
begin
   try
      E_WorkSheet.Columns['A:'+Alpha(ColCount)].AutoFit;
   except
      raise EWorkSheetError.Create('WorkSheet AutoFit Error.');
   end;
end;

procedure TWorkSheet.Activate;
begin
   try
      E_WorkSheet.Activate;
   except
      raise EWorkSheetError.Create('WorkSheet Activate Error.');
   end;
end;


procedure TWorkSheet.PrintPreview;
begin
   try
      E_WorkSheet.PrintPreview;
   except
      raise EWorkSheetError.Create('WorkSheet PrintPreview Error.');
   end;
end;

procedure TWorkSheet.SetHeader(strL, strC, strR: string);
begin
   try
      E_WorkSheet.PageSetUp.LeftHeader   := strL;
      E_WorkSheet.PageSetUp.CenterHeader := strC;
      E_WorkSheet.PageSetUp.RightHeader  := strR;
   except
      raise EWorkSheetError.Create('WorkSheet SetHeader Error.');
   end;
end;

procedure TWorkSheet.SetFooter(strL, strC, strR: string);
begin
   try
      E_WorkSheet.PageSetUp.LeftFooter   := strL;
      E_WorkSheet.PageSetUp.CenterFooter := strC;
      E_WorkSheet.PageSetUp.RightFooter  := strR;
   except
      raise EWorkSheetError.Create('WorkSheet SetFooter Error.');
   end;
end;

procedure TWorkSheet.GridToExcel(SG: TStringGrid);
var
   X,Y :integer;
begin
   try
      for X := 0 to SG.ColCount-1 do begin
         for Y := 0 to SG.RowCount-1 do begin
            Cells[X+1,Y+1] := SG.Cells[X,Y];
         end;
      end;
   except
      raise EWorkSheetError.Create('WorkSheet GridToExcel Error.');
   end;
end;

procedure TWorkSheet.ListToExcel(LV: TListView);
var
   X,Y : integer;
   RowIndex : integer;
begin
   RowIndex := 2;
   try
      for Y := 0 to LV.Items.Count-1 do begin
          { ListView`FbN{bNX^Cv̎ }
          if LV.Checkboxes  then begin
             if LV.Items[Y].Checked then begin
                Cells[1,RowIndex] := LV.Items[Y].Caption;
                for X := 0 to LV.Items[Y].SubItems.Count-1 do begin
                    Cells[X+2,RowIndex] := LV.Items[Y].SubItems[X];
                end;
                Inc(RowIndex);
             end;
          { ListView`FbN{bNX^CvłȂ }
          end else begin
             Cells[1,Y+2] := LV.Items[Y].Caption;
             for X := 0 to LV.Items[Y].SubItems.Count-1 do begin
                 Cells[X+2,Y+2] := LV.Items[Y].SubItems[X];
             end;
          end;
      end;
      for X := 0 to LV.Columns.Count-1 do begin
          Cells[X+1,1] := LV.Columns[X].Caption;
      end;
   except
      raise EWorkSheetError.Create('WorkSheet ListToExcel Error.');
   end;
end;

procedure TWorkSheet.MakeTableTemplate(ColumnRect, ItemRect: TRect; LineWidth: integer);
begin
   try
      //ACeɂĂ̌r쐬
      E_WorkSheet.Range[ExcelRange(ItemRect)].Borders[xlinsideVartical].LineStyle   := xlLS_Dot;        //itemc
      E_WorkSheet.Range[ExcelRange(ItemRect)].Borders[xlinsideHorizontal].LineStyle := xlLS_Continuous; //item
      //ACeɂĂ̌r쐬
      E_WorkSheet.Range[ExcelRange(ItemRect)].Borders[xlEdgeLeft].Weight   := LineWidth; //itemg
      E_WorkSheet.Range[ExcelRange(ItemRect)].Borders[xlEdgeTop].Weight    := LineWidth; //itemg
      E_WorkSheet.Range[ExcelRange(ItemRect)].Borders[xlEdgeBottom].Weight := LineWidth; //itemg
      E_WorkSheet.Range[ExcelRange(ItemRect)].Borders[xlEdgeRight].Weight  := LineWidth; //itemgE
      //JɂĂ̌r쐬
      E_WorkSheet.Range[ExcelRange(ColumnRect)].Borders[xlinsideVartical].LineStyle := 1;  //Columnc
      E_WorkSheet.Range[ExcelRange(ColumnRect)].Borders[xlEdgeLeft].Weight   := LineWidth; //Column
      E_WorkSheet.Range[ExcelRange(ColumnRect)].Borders[xlEdgeTop].Weight    := LineWidth; //Column
      E_WorkSheet.Range[ExcelRange(ColumnRect)].Borders[xlEdgeBottom].Weight := LineWidth; //Column
      E_WorkSheet.Range[ExcelRange(ColumnRect)].Borders[xlEdgeRight].Weight  := LineWidth; //ColumnE
   except
      raise EWorkSheetError.Create('WorkSheet MakeTableTemplate Error.');
   end;
end;

procedure TWorkSheet.MakeGraph(Rect: TRect; GraphKind: integer;
  HasLegend: Boolean; Title: string);
begin
   try
      E_Charts := E_WorkSheet.ChartObjects.Add(0,0,400,400);
      E_Charts.Left := 0;
      E_Charts.Top  := Rect.Bottom * 14; //Ƃ̾قTopʒuƂ
      E_Charts.Chart.ChartWizard(E_WorkSheet.Range[ExcelRange(Rect)],  // ͈͎̂̔w       //
                                                           GraphKind,  // ̂̎           //
                                                                   1,  // ̫̂ϯ[1`10]  //
                                                                   1,  // ްn̕[1`2] //
                                                                   1,  // ڍs̎w         //
                                                                   1,  // ڗ̎w         //
                                                           HasLegend,  // }̕\̗L     //
                                                               Title); // Title̎w          //
   except
      raise EWorkSheetError.Create('WorkSheet MakeGraph Error.');
   end;
end;




{ TExcel }

function TExcel.GetSheetsCount: integer;
begin
   Result := Length(WorkSheets);
end;

function TExcel.GetPrinterName: string;
begin
   Result := E_Application.ActivePrinter;
end;

function TExcel.GetHeight: integer;
begin
   Result := E_Excel.Height;
end;

procedure TExcel.SetHeight(Height: integer);
begin
   E_Excel.Height := Height;
end;

function TExcel.GetWidth: integer;
begin
   Result := E_Excel.Width;
end;

procedure TExcel.SetWidth(Width: integer);
begin
   E_Excel.Width := Width;
end;

function TExcel.GetTop: integer;
begin
   Result := E_Excel.Top;
end;

procedure TExcel.SetTop(Top: integer);
begin
   E_Excel.Top := Top;
end;

function TExcel.GetLeft: integer;
begin
   Result := E_Excel.Left;
end;

procedure TExcel.SetLeft(Left: integer);
begin
   E_Excel.Left := Left;
end;

constructor TExcel.Create(FileName:string);
var
   i:integer;
   Count:integer;
begin
   inherited Create;
   try
       E_Excel := CreateOleObject('Excel.Application');
       E_Application := E_Excel.Application;
       try
           //xlst@Cǉ
           if FileExists(FileName) then
              E_Application.WorkBooks.Open(FileName)
           else
              E_Application.WorkBooks.Add;

           //WorkBookɃANZX邽߂̕ϐ
           E_WorkBook  := E_Application.ActiveWorkbook;
           //ׂẴV[gi[
           Count := E_Application.Sheets.Count;
           SetLength(WorkSheets,Count);
           for i := 0 to Count-1 do begin
               WorkSheets[i] := TWorkSheet.Create( E_Application.Sheets[i+1] );
           end;
           //̌w
           //E_ActiveSheet.PageSetup.Orientation := xlPreviewWide;//[ݒ]
           //ɖs̎w(A1->1s)
           //E_ActiveSheet.PageSetup.PrintTitleRows := 'A1';
       except
           E_Application.DisplayAlerts := False;//bZ[W_CAO\[OFF]
           E_Application.Quit;
           E_Application := unAssigned;
           raise EExcelError.Create('Excel Create Error.');
       end;
   //CXg[ĂȂ
   except
      on EOleSysError do begin
         E_Excel := Unassigned;
         raise EExcelError.Create('Excel Create Error.');
      end;
   end;
   E_Application.Visible:= True;
end;

destructor TExcel.Destroy;
var
   i,Count:integer;
begin
   Count := Length(WorkSheets);
   for i := 0 to Count-1 do begin
       WorkSheets[i].Free;
   end;
   E_Application := UnAssigned;
   E_Excel       := Unassigned;
   E_WorkBook    := Unassigned;

   inherited Destroy;
end;

procedure TExcel.ExcelClose;
begin
   try
      E_Application.DisplayAlerts := False;//bZ[W_CAO̕\[OFF]
      E_Application.Quit;
      E_Application := unAssigned;
   except
      on E:Exception do begin
         raise EExcelError.Create('Excel Close Error. msg='+E.Message);
      end;
   end;
end;

procedure TExcel.SaveAs(FileName: string;FileFormatType:integer);
begin
   try
      E_WorkBook.SaveAs(FileName,FileFormatType);
   except
      on E:Exception do begin
         raise EExcelError.Create('Excel SaveToFile Error. msg='+E.Message);
      end;
   end;
end;







end.
